#include "demo.h"

// ˢ
void clear_white()
{
    epd_set_clear_color(EPD_COLOR, EPD_COLOR);
    epd_clear();
}

void fill_rect(epd_uint8_t bw, epd_uint8_t xw, epd_uint16_t x, epd_uint16_t y, epd_uint16_t width, epd_uint16_t height)
{
    epd_rect_t rect;
    
    rect.x = x;
    rect.y = y;
    rect.width = width;
    rect.height = height;
    
    epd_fill_color_with_rect(epd_fill_bw_ch, bw, rect);
    epd_fill_color_with_rect(epd_fill_xw_ch, xw, rect);
}

void demo_fill_rectangle()
{
    fill_rect(EPD_UNCOLOR, EPD_UNCOLOR, 0, 0, 200, 150); // ɫ
    fill_rect(EPD_COLOR, EPD_UNCOLOR, 200, 0, 200, 150); // ɫ
    fill_rect(EPD_UNCOLOR, EPD_COLOR, 0, 150, 200, 150); // ɫ
    fill_rect(EPD_COLOR, EPD_COLOR, 200, 150, 200, 150); // ɫ
}

#define my_buffer_width 200
#define my_buffer_height 150
#define my_buffer_size (my_buffer_width / 8 * my_buffer_height)

xdata epd_uint8_t w200h150buffer_bw[my_buffer_size];
xdata epd_uint8_t w200h150buffer_xw[my_buffer_size];

void clear_buffer(epd_uint8_t bw, epd_uint8_t xw)
{
    epd_uint16_t i;
    
    for (i = 0; i < my_buffer_size; i++)
    {
        w200h150buffer_bw[i] = bw;
        w200h150buffer_xw[i] = xw;
    }
}

void buffer_set_dot(epd_uint8_t bw, epd_uint8_t xw, epd_uint16_t x, epd_uint16_t y)
{
    if (x >= my_buffer_width || y >= my_buffer_height)
    {
        return;
    }
    
    if (bw == EPD_COLOR)
    {
        w200h150buffer_bw[(x + y * my_buffer_width) / 8] &= ~(0x80 >> (x % 8));
    }
    else
    {
        w200h150buffer_bw[(x + y * my_buffer_width) / 8] |= 0x80 >> (x % 8);
    }
    
    if (xw == EPD_COLOR)
    {
        w200h150buffer_xw[(x + y * my_buffer_width) / 8] &= ~(0x80 >> (x % 8));
    }
    else
    {
        w200h150buffer_xw[(x + y * my_buffer_width) / 8] |= 0x80 >> (x % 8);
    }
}

void draw_hline(epd_uint8_t bw, epd_uint8_t xw, int x, int y, int line_width) {
    int i;
    for (i = x; i < x + line_width; i++) {
        buffer_set_dot(bw, xw, i, y);
    }
}

void draw_line(epd_uint8_t bw, epd_uint8_t xw, int x0, int y0, int x1, int y1)
{
    int dx = x1 - x0 >= 0 ? x1 - x0 : x0 - x1;
    int sx = x0 < x1 ? 1 : -1;
    int dy = y1 - y0 <= 0 ? y1 - y0 : y0 - y1;
    int sy = y0 < y1 ? 1 : -1;
    int err = dx + dy;

    while((x0 != x1) && (y0 != y1)) {
        buffer_set_dot(bw, xw, x0, y0);
        if (2 * err >= dy) {     
            err += dy;
            x0 += sx;
        }
        if (2 * err <= dx) {
            err += dx; 
            y0 += sy;
        }
    }
}

void demo_draw_line()
{
    epd_rect_t rect;
    
    clear_buffer(EPD_COLOR, EPD_COLOR);
    
    draw_line(EPD_UNCOLOR, EPD_COLOR, 0, 0, 200, 150);
    draw_line(EPD_UNCOLOR, EPD_UNCOLOR, 200, 0, 0, 150);
    
    rect.x = 200;
    rect.y = 150;
    rect.width = my_buffer_width;
    rect.height = my_buffer_height;
    
    epd_fill_image_with_rect(epd_fill_bw_ch, w200h150buffer_bw, rect);
    epd_fill_image_with_rect(epd_fill_xw_ch, w200h150buffer_xw, rect);
    
    
    clear_buffer(EPD_UNCOLOR, EPD_COLOR);
    
    draw_line(EPD_UNCOLOR, EPD_UNCOLOR, 0, 0, 200, 150);
    draw_line(EPD_COLOR, EPD_COLOR, 200, 0, 0, 150);
    
    rect.x = 0;
    rect.y = 150;
    rect.width = my_buffer_width;
    rect.height = my_buffer_height;
    
    epd_fill_image_with_rect(epd_fill_bw_ch, w200h150buffer_bw, rect);
    epd_fill_image_with_rect(epd_fill_xw_ch, w200h150buffer_xw, rect);
}

void fill_circle(epd_uint8_t bw, epd_uint8_t xw, int x, int y, int radius)
{
    int x_pos = -radius;
    int y_pos = 0;
    int err = 2 - 2 * radius;
    int e2;

    do {
        buffer_set_dot(bw, xw, x - x_pos, y + y_pos);
        buffer_set_dot(bw, xw, x + x_pos, y + y_pos);
        buffer_set_dot(bw, xw, x + x_pos, y - y_pos);
        buffer_set_dot(bw, xw, x - x_pos, y - y_pos);
        draw_hline(bw, xw, x + x_pos, y + y_pos, 2 * (-x_pos) + 1);
        draw_hline(bw, xw, x + x_pos, y - y_pos, 2 * (-x_pos) + 1);
        e2 = err;
        if (e2 <= y_pos) {
            err += ++y_pos * 2 + 1;
            if(-x_pos == y_pos && e2 <= x_pos) {
                e2 = 0;
            }
        }
        if(e2 > x_pos) {
            err += ++x_pos * 2 + 1;
        }
    } while(x_pos <= 0);
}

void demo_fill_circle()
{
    epd_rect_t rect;
    
    clear_buffer(EPD_UNCOLOR, EPD_UNCOLOR);
    fill_circle(EPD_COLOR, EPD_COLOR, 100, 75, 64);
    
    rect.x = 0;
    rect.y = 0;
    rect.width = my_buffer_width;
    rect.height = my_buffer_height;
    
    epd_fill_image_with_rect(epd_fill_bw_ch, w200h150buffer_bw, rect);
    epd_fill_image_with_rect(epd_fill_xw_ch, w200h150buffer_xw, rect);
}

void delay_100ms(int s)
{
    while (s--)
    {
        epd_hal_delay_ms(100);
    }
}

void demo_draw_image()
{
    // 400 x 300
    epd_fill_image(epd_fill_bw_ch, pixels_for_color_0x00000000);
    epd_fill_image(epd_fill_xw_ch, pixels_for_color_0x00FF0000);
    
    epd_refresh();
}

void main(void)
{
    epd_hal_io_init();
    epd_hal_spi_init();
    epd_hal_delay_init();
    epd_init();
    
    clear_white();
    
    demo_fill_rectangle();
    demo_draw_line();
    demo_fill_circle();
    
    epd_refresh();
    
    delay_100ms(30);
    
    demo_draw_image();
    
    epd_sleep();
    
    while (1)
    {
    }
}
